'use strict';

// # Globbing
// for performance reasons we're only matching one level down:
// 'test/spec/{,*/}*.js'
// use this if you want to recursively match all subfolders:
// 'test/spec/**/*.js'

module.exports = function (grunt) {

  // Load grunt tasks automatically
  require('load-grunt-tasks')(grunt);

  // Time how long tasks take. Can help when optimizing build times
  require('time-grunt')(grunt);

  // Configurable paths
  var config = {
    app: 'app.safariextension',
    dist: 'dist.safariextension'
  };

  grunt.initConfig({

    // Project settings
    config: config,

    // Watches files for changes and runs tasks based on the changed files
    watch: {
      bower: {
        files: ['bower.json'],
        tasks: ['bower:install', 'wiredep']
      },<% if (coffee) { %>
      coffee: {
        files: ['<%%= config.app %>/scripts/{,*/}*.{coffee,litcoffee,coffee.md}'],
        tasks: ['coffee:app']
      },<%} else { %>
      js: {
        files: ['<%%= config.app %>/scripts/{,*/}*.js'],
        tasks: ['jshint']
      },<% } %><% if (compass) { %>
      compass: {
        files: ['<%%= config.app %>/styles/{,*/}*.{scss,sass}'],
        tasks: ['compass:app']
      },<% } %>
      gruntfile: {
        files: ['Gruntfile.js']
      },
      styles: {
        files: ['<%%= config.app %>/styles/{,*/}*.css'],
        tasks: [],
        options: {
        }
      }
    },<% if (coffee) { %>

    // Compiles CoffeeScript to JavaScript
    coffee: {
      app: {
        files: [{
          expand: true,
          cwd: '<%%= config.app %>/scripts',
          src: '{,*/}*.{coffee,litcoffee,coffee.md}',
          dest: '<%%= config.app %>/scripts',
          ext: '.js'
        }]
      },
      dist: {
        files: [{
          expand: true,
          cwd: '<%%= config.app %>/scripts',
          src: '{,*/}*.{coffee,litcoffee,coffee.md}',
          dest: '<%%= config.app %>/scripts',
          ext: '.js'
        }]
      },
      test: {
        files: [{
          expand: true,
          cwd: 'test/spec',
          src: '{,*/}*.coffee',
          dest: './spec',
          ext: '.js'
        }]
      }
    },<% } %><% if (compass) { %>

     // Compiles Sass to CSS and generates necessary files if requested
    compass: {
      options: {
        sassDir: '<%%= config.app %>/styles',
        cssDir: '<%%= config.app %>/styles',
        generatedImagesDir: '<%%= config.app %>/images/generated',
        imagesDir: '<%%= config.app %>/images',
        javascriptsDir: '<%%= config.app %>/scripts',
        fontsDir: '<%%= config.app %>/styles/fonts',
        httpImagesPath: '/images',
        httpGeneratedImagesPath: '/images/generated',
        httpFontsPath: '/styles/fonts',
        relativeAssets: false,
        assetCacheBuster: false
      },
      app: {
        options: {
          debugInfo: true
        }
      },
      dist: {
        options: {
          force: true
        }
      },
      test: {
      }
    },<% } %>

    // Install Bower packages
    bower: {
      install: {
      }
    },

    // Automatically inject Bower components into the HTML file
    wiredep: {
      app: {
        src: [
          '<%%= config.app %>/options.html'
        ]
      },
      sass: {
        src: ['<%%= config.app %>/styles/{,*/}*.{scss,sass}'],
        ignorePath: '<%%= config.app %>/bower_components/'
      }
    },

    uglify: {
      dist: {
        files: {
        }
      }
    },

    // Reads HTML for usemin blocks to enable smart builds that automatically
    // concat, minify and revision files. Creates configurations in memory so
    // additional tasks can operate on them
    useminPrepare: {
      options: {
        dest: '<%%= config.dist %>'
      },
      html: [
        '<%%= config.app %>/global.html',
        '<%%= config.app %>/options.html'
      ]
    },

    // Performs rewrites based on rev and the useminPrepare configuration
    usemin: {
      options: {
        assetsDirs: ['<%%= config.dist %>', '<%%= config.dist %>/images']
      },
      html: ['<%%= config.dist %>/{,*/}*.html'],
      css: ['<%%= config.dist %>/styles/{,*/}*.css']
    },

    // The following *-min tasks produce minifies files in the dist folder
    imagemin: {
      icon: {
        files: {
          '<%%= config.dist %>/icon.png': '<%%= config.app %>/icon.png'
        }
      },
      dist: {
        files: [{
          expand: true,
          cwd: '<%%= config.app %>/images',
          src: '{,*/}*.{gif,jpeg,jpg,png}',
          dest: '<%%= config.dist %>/images'
        }]
      }
    },

    // Empties folders to start fresh
    clean: {
      app: {
      },
      dist: {
        files: [{
          dot: true,
          src: [
            '<%%= config.dist %>/*',
            '!<%%= config.dist %>/.git*'
          ]
        }]
      }
    },

    // Make sure code styles are up to par and there are no obvious mistakes
    jshint: {
      options: {
        jshintrc: '.jshintrc',
        reporter: require('jshint-stylish')
      },
      all: [
        'Gruntfile.js',
        '<%%= config.app %>/scripts/{,*/}*.js',
        '!<%%= config.app %>/scripts/vendor/*',
        'test/spec/{,*/}*.js'
      ]
    },

    // Copies remaining files to places other tasks can use
    copy: {
      dist: {
        files: [{
          expand: true,
          dot: true,
          cwd: '<%%= config.app %>',
          dest: '<%%= config.dist %>',
          src: [
            '*.{ico,txt}',
            'images/{,*/}*.{webp,gif}',
            '{,*/}*.html',
            'styles/fonts/{,*/}*.*',
            '{,*/}*.plist',
          ]
        }]
      }
    },

    // Run some tasks in parallel to speed up build process
    concurrent: {
      app: [<% if (coffee) { %>
        'coffee:app',<% } if (compass) { %>
        'compass:app',<% } %>
      ],
      dist: [<% if (coffee) { %>
        'coffee:dist',<% } %>
        'imagemin',<% if (compass) { %>
        'compass:dist',<% } %>
      ],
      test: [<% if (coffee) { %>
        'coffee:test',<% } if (compass) { %>
        'compass:test',<% } %>
      ]
    },
  });

  grunt.registerTask('debug', function () {
    grunt.task.run([
      'jshint',
      'concurrent:app',
      'watch'
    ]);
  });

  grunt.registerTask('build', [
    'clean:dist',
    'concurrent:dist',
    'useminPrepare',
    'concat',
    'cssmin',
    'uglify',
    'copy',
    'usemin'
  ]);

  grunt.registerTask('default', [
    'jshint',
    'build'
  ]);

};
